package ru.dbotthepony.mc.otm.storage;


import net.minecraft.MethodsReturnNonnullByDefault;
import ru.dbotthepony.mc.otm.core.Fraction;

import javax.annotation.ParametersAreNonnullByDefault;
import java.math.BigDecimal;
import java.util.List;
import java.util.UUID;

@MethodsReturnNonnullByDefault
@ParametersAreNonnullByDefault
public interface IStorageView<T extends IStorageStack> extends IStorageTrigger<T> {
	/**
	 * @param id identifier of object
	 * @return stored object (not a copy). Do not edit it.
	 */
	T getStack(UUID id);

	/**
	 * @param id identifier of object to extract
	 * @param amount amount of units to extract
	 * @param simulate whenever to simulate the action or not
	 * @return copy of object, with amount of units actually extracted
	 */
	T extractStack(UUID id, Fraction amount, boolean simulate);

	/**
	 * Designed for views, for extraction with less computation overhead caused by
	 * copying object extracted
	 *
	 * @param id identifier of object to extract
	 * @param amount desired amount to extract
	 * @param simulate whenever to simulate the action or not
	 * @return amount extracted
	 */
	default Fraction extractStackCount(UUID id, Fraction amount, boolean simulate) {
		return extractStack(id, amount, simulate).getCount();
	}

	List<IStorageTuple<T>> getStacks();

	default boolean addListenerAuto(IStorageListener<T> listener) {
		if (addListener(listener)) {
			for (var stack : getStacks()) {
				listener.addObject(stack.stack(), stack.id(), this);
			}

			return true;
		}

		return false;
	}

	default boolean removeListenerAuto(IStorageListener<T> listener) {
		if (removeListener(listener)) {
			for (var stack : getStacks()) {
				listener.removeObject(stack.id());
			}

			return true;
		}

		return false;
	}
}
