package ru.dbotthepony.mc.otm.storage;

import net.minecraft.MethodsReturnNonnullByDefault;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.Level;
import net.minecraftforge.common.util.LazyOptional;
import ru.dbotthepony.mc.otm.AbstractGrid;
import ru.dbotthepony.mc.otm.OverdriveThatMatters;
import ru.dbotthepony.mc.otm.capability.IStorageGridCell;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;

import javax.annotation.ParametersAreNonnullByDefault;
import java.util.*;

@MethodsReturnNonnullByDefault
@ParametersAreNonnullByDefault
public class StorageGrid extends AbstractGrid<IStorageGridCell> {
	private final HashMap<Class<? extends IStorageStack>, VirtualComponent<? extends IStorageStack>> virtual_components = new HashMap<>();

	public StorageGrid() {
		super(MatteryCapability.STORAGE_CELL);
	}

	public <T extends IStorageStack> T insertObject(Class<T> type, T object, boolean simulate) {
		return getVirtualComponent(type).insertStack(object, simulate);
	}

	/**
	 * @param type Class, representing identity of stored object
	 * @param <T> The identity itself
	 * @return a virtual IStorageComponent<T>, or null if storage grid contain no view for provided type
	 */
	@SuppressWarnings("unchecked")
	public <T extends IStorageStack> VirtualComponent<T> getVirtualComponent(final Class<T> type) {
		return (VirtualComponent<T>) virtual_components.computeIfAbsent(type, (k) -> new VirtualComponent<>(type));
	}

	public <T extends IStorageStack> void add(IStorageIdentity<T> identity) {
		getVirtualComponent(identity.storageIdentity()).add(identity);
	}

	public <T extends IStorageStack> void remove(IStorageIdentity<T> identity) {
		getVirtualComponent(identity.storageIdentity()).remove(identity);
	}

	@Override
	protected void onAdded(IStorageGridCell cell) {
		cell.setStorageGrid(this);

		for (var identity : cell.getComponents()) {
			add(identity);
		}
	}

	@Override
	protected void onRemoved(IStorageGridCell cell) {
		cell.setStorageGrid(null);

		for (var identity : cell.getComponents()) {
			remove(identity);
		}
	}

	public void detach(IStorageGridCell cell) {
		if (cells.contains(cell)) {
			for (var identity : cell.getComponents()) {
				remove(identity);
			}
		}
	}

	public void attach(IStorageGridCell cell) {
		if (cells.contains(cell)) {
			for (var identity : cell.getComponents()) {
				add(identity);
			}
		}
	}

	public static void scheduleDiscoverNeighbours(IStorageGridCell cell, Level level, BlockPos pos) {
		OverdriveThatMatters.tickUntil(level, () -> !cell.isValidStorageCell() || cell.connectOrCreateStorageGrid(level, pos, true));
	}
}
