package ru.dbotthepony.mc.otm.client.screen

import com.mojang.blaze3d.vertex.PoseStack
import net.minecraft.ChatFormatting
import net.minecraft.client.Minecraft
import net.minecraft.network.chat.Component
import net.minecraft.network.chat.TranslatableComponent
import net.minecraft.world.entity.player.Inventory
import ru.dbotthepony.mc.otm.Registry
import ru.dbotthepony.mc.otm.android.AndroidResearch
import ru.dbotthepony.mc.otm.android.AndroidResearchType
import ru.dbotthepony.mc.otm.capability.MatteryCapability
import ru.dbotthepony.mc.otm.capability.android.AndroidCapabilityPlayer
import ru.dbotthepony.mc.otm.capability.android.IAndroidCapability
import ru.dbotthepony.mc.otm.client.render.RGBAColor
import ru.dbotthepony.mc.otm.client.render.RenderHelper
import ru.dbotthepony.mc.otm.client.screen.panels.*
import ru.dbotthepony.mc.otm.client.screen.widget.PowerGaugePanel
import ru.dbotthepony.mc.otm.menu.AndroidStationMenu
import java.util.*

class AndroidStationScreen constructor(p_97741_: AndroidStationMenu, p_97742_: Inventory, p_97743_: Component) :
	MatteryScreen<AndroidStationMenu>(p_97741_, p_97742_, p_97743_) {

	internal inner class AndroidResearchButton(parent: EditablePanel?, private val node: AndroidResearch) :
		EditablePanel(
			this@AndroidStationScreen,
			parent,
			0f,
			0f,
			BUTTON_SIZE.toFloat(),
			BUTTON_SIZE.toFloat()
		) {

		init {
			setDockMargin(2f, 2f, 2f, 2f)
		}

		override fun innerRender(stack: PoseStack, mouse_x: Float, mouse_y: Float, flag: Float) {
			Minecraft.getInstance().player!!.getCapability(MatteryCapability.ANDROID)
				.ifPresent { _cap: IAndroidCapability? ->
					if (_cap is AndroidCapabilityPlayer) {
						if (node.isResearched) {
							RESEARCHED.setSystemColor()
						} else if (node.canResearch()) {
							CAN_BE_RESEARCHED.setSystemColor()
						} else {
							CAN_NOT_BE_RESEARCHED.setSystemColor()
						}

						val icon = node.icon

						if (icon != null) {
							icon.render(stack, 0f, 0f, width, height)
						} else {
							RenderHelper.drawRect(stack, 0f, 0f, width, height)
						}

						val text = node.iconText

						if (text != null) {
							font.drawShadow(stack, text, width - font.width(text), height - font.lineHeight, -0x1)
						}
					}
				}
		}

		override fun mouseClickedInner(mouse_x: Double, mouse_y: Double, flag: Int): Boolean {
			node.researchClient()
			return true
		}

		override fun innerRenderTooltips(stack: PoseStack, mouse_x: Float, mouse_y: Float, flag: Float): Boolean {
			if (isHovered) {
				val list = ArrayList(
					node.tooltip
				)

				if (node.isResearched) {
					list.add(TranslatableComponent("otm.android_station.research.researched").withStyle(ChatFormatting.DARK_AQUA))
				} else if (node.canResearch()) {
					list.add(TranslatableComponent("otm.android_station.research.can_be_researched").withStyle(ChatFormatting.DARK_GREEN))
				} else {
					list.add(TranslatableComponent("otm.android_station.research.can_not_be_researched").withStyle(ChatFormatting.DARK_RED))
				}

				renderComponentTooltip(stack, list, mouse_x.toInt(), mouse_y.toInt())
			}

			return isHovered
		}
	}

	private val rows = arrayOfNulls<EditablePanel>(100)
	private val seen: MutableSet<AndroidResearchType<*>> = HashSet()
	private var nextX = 0f
	private val rowsWidth = FloatArray(100)
	private val createdButtons = Array(100) { arrayOfNulls<AndroidResearchButton>(1000) }
	private val createdButtonsIdx = IntArray(100)

	private fun dive(cap: AndroidCapabilityPlayer, research: AndroidResearchType<*>, level: Int) {
		if (seen.contains(research)) return
		seen.add(research)

		if (rows[level] == null) {
			rows[level] = object : EditablePanel(this@AndroidStationScreen, canvas, 0f, (level * 22).toFloat(), 10000f, 22f) {
				override fun mouseClickedInner(mouse_x: Double, mouse_y: Double, flag: Int) = false
				override fun mouseReleasedInner(mouse_x: Double, mouse_y: Double, flag: Int) = false
				override fun mouseDraggedInner(mouse_x: Double, mouse_y: Double, flag: Int, drag_x: Double, drag_y: Double) = false
			}
		}

		val row = rows[level]
		val button = AndroidResearchButton(row, cap.getResearch(research))

		button.setPos(nextX + rowsWidth[level], 2f)

		createdButtons[level][createdButtonsIdx[level]] = button
		createdButtonsIdx[level]++
		rowsWidth[level] += 22f

		for (_research in research.unlocks) {
			dive(cap, _research, level + 1)
		}

		if (level > 0) {
			for (_research in research.prerequisites) {
				dive(cap, _research, level - 1)
			}
		}
	}

	private var canvas: DraggableCanvasPanel? = null
	private var research: FramePanel? = null

	private fun openResearchTree() {
		val window = minecraft!!.window

		research = FramePanel(this, null, 0f, 0f, window.guiScaledWidth * 0.8f, window.guiScaledHeight * 0.8f, TranslatableComponent("otm.gui.android_research"))

		canvas = object : DraggableCanvasPanel(this@AndroidStationScreen, research, width = (GRID_WIDTH * 22).toFloat(), height = 0f) {
			override fun innerRender(stack: PoseStack, mouse_x: Float, mouse_y: Float, flag: Float) {
				RenderHelper.setDrawColor(RGBAColor.BLACK)
				RenderHelper.drawRect(stack, 0f, 0f, width, height)
			}
		}

		minecraft?.player?.getCapability(MatteryCapability.ANDROID)?.ifPresent {
			if (it !is AndroidCapabilityPlayer)
				return@ifPresent

			Arrays.fill(rows, null)
			nextX = 0f

			for (research in Registry.ANDROID_RESEARCH.values) {
				if (research.prerequisites.size == 0) {
					dive(it, research, 0)

					var max = 0f

					for (v in rowsWidth)
						max = Math.max(max, v)

					for (button_list in createdButtons) {
						var count = 0

						for (i in button_list.indices) {
							if (button_list[i] == null) {
								count = i
								break
							}
						}

						if (count > 0) {
							var thisX = nextX + max / 2f - count * 22f / 2f

							for (i in 0 until count) {
								button_list[i]!!.setPos(thisX, 2f)
								thisX += 22f
							}
						}
					}

					for (v in createdButtons)
						Arrays.fill(v, null)

					nextX += max
					Arrays.fill(rowsWidth, 0f)
					Arrays.fill(createdButtonsIdx, 0)
				}
			}

			seen.clear()
		}

		canvas!!.dock = Dock.FILL

		val bottom = EditablePanel(this, research, 0f, 0f, 0f, 20f)
		val close = ButtonPanel(this, bottom, 0f, 0f, 90f, 20f, TranslatableComponent("otm.container.matter_panel.close"))

		bottom.dock = Dock.BOTTOM

		close.dock = Dock.LEFT
		close.bindOnPress { research!!.remove() }

		bottom.setDockMargin(0f, 0f, 4f, 4f)
		canvas!!.setDockMargin(4f, 4f, 4f, 4f)

		research!!.toScreenCenter()
		addPanel(research!!)
	}

	override fun resize(minecraft: Minecraft, p_96576_: Int, p_96577_: Int) {
		super.resize(minecraft, p_96576_, p_96577_)

		val research = research

		if (research != null) {
			val window = minecraft.window
			research.setSize(window.guiScaledWidth * 0.8f, window.guiScaledHeight * 0.8f)
			research.toScreenCenter()
		}
	}

	override fun makeMainFrame(): FramePanel {
		val frame = super.makeMainFrame()!!

		PowerGaugePanel(this, frame, menu.battery_widget, LEFT_MARGIN, GAUGE_TOP_WITH_SLOT)
		SlotPanel(this, frame, menu.battery_slot, LEFT_MARGIN, SLOT_TOP_UNDER_GAUGE)
		SlotPanel(this, frame, menu.androidBattery, 38f, 17f)

		val button = ButtonPanel(this, frame, 38f, 69f, 124f, 20f, TranslatableComponent("otm.gui.android_research"))
		button.bindOnPress(this::openResearchTree)

		return frame
	}

	companion object {
		const val GRID_WIDTH = 5
		const val BUTTON_SIZE = 18
		val RESEARCHED = RGBAColor(150, 150, 200)
		val CAN_BE_RESEARCHED = RGBAColor(150, 200, 150)
		val CAN_NOT_BE_RESEARCHED = RGBAColor(200, 150, 150)
	}
}